package com.localtransfer;

import android.animation.ObjectAnimator;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.LinearLayout;

import androidx.core.content.FileProvider;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.sql.Timestamp;

public class DownloadFile extends Transfer {

    public String href;

    public String mime;

    public String type;

    public Uri uri;

    private Button button;

    public DownloadFile(Integer id) {
        this.id = id;
        instances.add(this);
        info = "Download complete";
        state = STATE_NOT_REQUESTED;
        drawable = R.drawable.ic_download_24;
    }

    @Override
    public void startTransfer() {
        super.startTransfer();

        startedTime = new Timestamp(System.currentTimeMillis());

        state = STATE_RUNNING;

        buttonProgressStart();

        if (uri == null && android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
            ContentValues values = new ContentValues();
            values.put(MediaStore.MediaColumns.DISPLAY_NAME, name);
            values.put(MediaStore.MediaColumns.MIME_TYPE, mime);
            values.put(MediaStore.MediaColumns.RELATIVE_PATH, Transfer.shared_storage);
            uri = resolver.insert(MediaStore.Downloads.EXTERNAL_CONTENT_URI, values);
        }
        try {
            downloadFile();
        } catch (IOException | NullPointerException e) {
            final String ExceptionName = e.getClass().getSimpleName();
            final String ExceptionMess = e.getMessage();
            e.printStackTrace();

            state = Transfer.STATE_FAILED;
            progressEnd();
            buttonProgressEnd();

            if(ExceptionName != null && ExceptionMess != null) {
                errorSnackbar(ExceptionName + ": " + ExceptionMess);
            }

        }
    }

    public boolean exist(Context context) {
        if (!Transfer.use_shared_storage){
            File file = new File(activity.getFilesDir(), name);

            uri = FileProvider.getUriForFile(activity, activity.getPackageName(), file);

            if (file.exists() && file.length() == size)
                return true;
        }
        else {
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
                Cursor cursor = context.getContentResolver().query(MediaStore.Downloads.EXTERNAL_CONTENT_URI, null, null, null, null);
                while (cursor.moveToNext()) {
                    String MediaStore_File_name = cursor.getString(cursor.getColumnIndexOrThrow(MediaStore.Downloads.DISPLAY_NAME));
                    long MediaStore_File_size = cursor.getLong(cursor.getColumnIndexOrThrow(MediaStore.Downloads.SIZE));
                    if(MediaStore_File_name.equals(name) && MediaStore_File_size == size) {
                        int cursor_id = cursor.getInt(cursor.getColumnIndexOrThrow(MediaStore.Downloads._ID));
                        uri = ContentUris.withAppendedId(MediaStore.Downloads.EXTERNAL_CONTENT_URI, cursor_id);
                        return true;
                    }
                }
            }
            else {
                String save_location = null;
                if (mime.equals("text/plain"))
                    save_location = context.getCacheDir().toString();
                else
                    save_location = Environment.getExternalStorageDirectory() + "/" + Transfer.shared_storage;
                new File(save_location).mkdirs();
                File file = new File(save_location, name);
                try {
                    file.createNewFile();
                } catch (IOException e) {
                    final String ExceptionName = e.getClass().getSimpleName();
                    final String ExceptionMess = e.getMessage();
                    e.printStackTrace();

                    if(ExceptionName != null && ExceptionMess != null) {
                        errorSnackbar(ExceptionName + ": " + ExceptionMess);
                    }
                }
                uri = Uri.fromFile(file);
                if (file.exists() && file.length() == size)
                    return true;
            }
        }
        return false;
    }

    public void setButton(Button button) {
        this.button = button;
        if (this.state == STATE_STANDBY) {
            buttonProgressStart();
        }
    }

    public InputStream getThumbnail() {
        InputStream thumbnail = null;
        URL url;
        try {
            String query = URLEncoder.encode(href, StandardCharsets.UTF_8.toString());
            String serverUrl = root + "/thumbnail.php?file=" + query;
            url = new URL(protocol, host, port, serverUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(2000);

            conn.connect();

            thumbnail = conn.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return thumbnail;
    }

    public void downloadFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;
        int maxBufferSize = 1 * 1024 * 1024;
        byte[] buffer = new byte[maxBufferSize];
        int bufferLength;
        loaded = 0;

        String[] parts = href.split("/");
        String path = "";
        for(int i=0; i< parts.length - 1; i++) {
            path = path + parts[i] + "/";
        }
        String query = URLEncoder.encode(parts[parts.length - 1], StandardCharsets.UTF_8.toString());
        url = new URL(protocol, host, port, path + query.replace("+", "%20"));
        conn = (HttpURLConnection) url.openConnection();
        conn.setDoOutput(true);
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        Log.d("URL", url.toString());

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            OutputStream fileOutput = activity.getContentResolver().openOutputStream(uri);
            InputStream in = conn.getInputStream();

            while ((bufferLength = in.read(buffer)) > 0) {
                fileOutput.write(buffer, 0, bufferLength);
                loaded+= bufferLength;
                percent = ((loaded * 100) / size);
                progressUpdateDelay();
                buttonProgressUpdate();
            }
            fileOutput.close();

            conn.disconnect();

            message = "File " + name + " successful download";
            state = Transfer.STATE_SUCCESS;
            progressEnd();
            buttonProgressEnd();

        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();
    }

    public String deleteFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;

        String message;

        String query = URLEncoder.encode(name, StandardCharsets.UTF_8.toString());
        String serverUrl = root + "/delete.php?file=" + query;
        url = new URL(protocol, host, port, serverUrl);
        conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            message = "File " + name + " successful deleted";
        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();

        return message;
    }

    public void buttonProgressStart() {
        activity.runOnUiThread(() -> {
            final Drawable image = Transfer.activity.getDrawable(R.drawable.ic_spinner_rotate);

            if(button != null) {
                int h = image.getIntrinsicHeight();
                int w = image.getIntrinsicWidth();
                image.setBounds(0, 0, w, h);
                button.setCompoundDrawables(button.getCompoundDrawables()[0], null, image, null);
                ObjectAnimator anim = ObjectAnimator.ofInt(image, "level", 0, 10000);
                anim.setDuration(1000);
                anim.setRepeatCount(Animation.INFINITE);
                anim.setInterpolator(new LinearInterpolator());
                anim.start();
            }
        });
    }

    private void buttonProgressUpdate(){
        activity.runOnUiThread(() -> {
            if (app_started && button != null)
                button.setText(String.format("Download in progress %d%%", percent));
        });
    }

    public void buttonProgressEnd() {
        activity.runOnUiThread(() -> {
            if (button != null) {
                final LinearLayout layout = (LinearLayout) button.getParent();
                layout.findViewById(R.id.file_view).setVisibility(LinearLayout.VISIBLE);
                layout.findViewById(R.id.file_share).setVisibility(LinearLayout.VISIBLE);
                button.setVisibility(LinearLayout.GONE);
                button.setEnabled(true);

                button.setText(activity.getString(R.string.file_download));
                button.setCompoundDrawables(button.getCompoundDrawables()[0], null, null, null);
            }
        });
    }
}
